@@ -53,13 +53,19 @@ import (
 // BuildACI takes a Docker layer, converts it to ACI and returns its output
 // path and its converted ImageManifest.
 type Docker2ACIBackend interface {
-	GetImageInfo(dockerUrl string) ([]string, *common.ParsedDockerURL, error)
-	BuildACI(layerIDs []string, dockerURL *common.ParsedDockerURL, outputDir string, tmpBaseDir string, compression common.Compression) ([]string, []*schema.ImageManifest, error)
+	// GetImageInfo, given the url for a docker image, will return the
+	// following:
+	// - []string: an ordered list of all layer hashes
+	// - string: a unique identifier for this image, like a hash of the manifest
+	// - *common.ParsedDockerURL: a parsed docker URL
+	// - error: an error if one occurred
+	GetImageInfo(dockerUrl string) ([]string, string, *common.ParsedDockerURL, error)
+	BuildACI(layerIDs []string, manhash string, dockerURL *common.ParsedDockerURL, outputDir string, tmpBaseDir string, compression common.Compression) ([]string, []*schema.ImageManifest, error)
 }
 
 // GenerateACI takes a Docker layer and generates an ACI from it.
-func GenerateACI(layerNumber int, layerData types.DockerImageData, dockerURL *common.ParsedDockerURL, outputDir string, layerFile *os.File, curPwl []string, compression common.Compression, debug log.Logger) (string, *schema.ImageManifest, error) {
-	manifest, err := GenerateManifest(layerData, dockerURL, debug)
+func GenerateACI(layerNumber int, manhash string, layerData types.DockerImageData, dockerURL *common.ParsedDockerURL, outputDir string, layerFile *os.File, curPwl []string, compression common.Compression, debug log.Logger) (string, *schema.ImageManifest, error) {
+	manifest, err := GenerateManifest(layerData, manhash, dockerURL, debug)
 	if err != nil {
 		return "", nil, fmt.Errorf("error generating the manifest: %v", err)
 	}
@@ -104,13 +110,13 @@ func GenerateACI22LowerLayer(dockerURL *common.ParsedDockerURL, layerDigest stri
 	return aciPath, manifest, nil
 }
 
-func GenerateACI22TopLayer(dockerURL *common.ParsedDockerURL, imageConfig *typesV2.ImageConfig, layerDigest string, outputDir string, layerFile *os.File, curPwl []string, compression common.Compression, lowerLayers []*schema.ImageManifest, debug log.Logger) (string, *schema.ImageManifest, error) {
+func GenerateACI22TopLayer(dockerURL *common.ParsedDockerURL, manhash string, imageConfig *typesV2.ImageConfig, layerDigest string, outputDir string, layerFile *os.File, curPwl []string, compression common.Compression, lowerLayers []*schema.ImageManifest, debug log.Logger) (string, *schema.ImageManifest, error) {
 	aciName := fmt.Sprintf("%s/%s-%s", dockerURL.IndexURL, dockerURL.ImageName, layerDigest)
 	sanitizedAciName, err := appctypes.SanitizeACIdentifier(aciName)
 	if err != nil {
 		return "", nil, err
 	}
-	manifest, err := GenerateManifestV22(sanitizedAciName, dockerURL, imageConfig, layerDigest, lowerLayers, debug)
+	manifest, err := GenerateManifestV22(sanitizedAciName, manhash, layerDigest, dockerURL, imageConfig, lowerLayers, debug)
 	if err != nil {
 		return "", nil, err
 	}
@@ -166,9 +172,45 @@ func generateEPCmdAnnotation(dockerEP, dockerCmd []string) (string, string, erro
 	return entrypointAnnotation, cmdAnnotation, nil
 }
 
+// setLabel sets the label entries associated with non-empty key
+// to the single non-empty value. It replaces any existing values
+// associated with key.
+func setLabel(labels map[appctypes.ACIdentifier]string, key, val string) {
+	if key != "" && val != "" {
+		labels[*appctypes.MustACIdentifier(key)] = val
+	}
+}
+
+// setOSArch translates the given OS and architecture strings into
+// the compatible with application container specification and sets
+// the respective label entries.
+//
+// Returns an error if label translation fails.
+func setOSArch(labels map[appctypes.ACIdentifier]string, os, arch string) error {
+	// Translate arch tuple into the appc arch tuple.
+	appcOS, appcArch, err := appctypes.ToAppcOSArch(os, arch, "")
+	if err != nil {
+		return err
+	}
+
+	// Set translated labels.
+	setLabel(labels, "os", appcOS)
+	setLabel(labels, "arch", appcArch)
+	return nil
+}
+
+// setAnnotation sets the annotation entries associated with non-empty
+// key to the single non-empty value. It replaces any existing values
+// associated with key.
+func setAnnotation(annotations *appctypes.Annotations, key, val string) {
+	if key != "" && val != "" {
+		annotations.Set(*appctypes.MustACIdentifier(key), val)
+	}
+}
+
 // GenerateManifest converts the docker manifest format to an appc
 // ImageManifest.
-func GenerateManifest(layerData types.DockerImageData, dockerURL *common.ParsedDockerURL, debug log.Logger) (*schema.ImageManifest, error) {
+func GenerateManifest(layerData types.DockerImageData, manhash string, dockerURL *common.ParsedDockerURL, debug log.Logger) (*schema.ImageManifest, error) {
 	dockerConfig := layerData.Config
 	genManifest := &schema.ImageManifest{}
 
@@ -195,42 +237,24 @@ func GenerateManifest(layerData types.DockerImageData, dockerURL *common.ParsedD
 	labels := make(map[appctypes.ACIdentifier]string)
 	parentLabels := make(map[appctypes.ACIdentifier]string)
 
-	addLabel := func(key, val string) {
-		if key != "" && val != "" {
-			labels[*appctypes.MustACIdentifier(key)] = val
-		}
-	}
+	setLabel(labels, "layer", layerData.ID)
+	setLabel(labels, "version", dockerURL.Tag)
 
-	addParentLabel := func(key, val string) {
-		if key != "" && val != "" {
-			parentLabels[*appctypes.MustACIdentifier(key)] = val
-		}
-	}
+	setOSArch(labels, layerData.OS, layerData.Architecture)
+	setOSArch(parentLabels, layerData.OS, layerData.Architecture)
 
-	addAnno := func(key, val string) {
-		if key != "" && val != "" {
-			annotations.Set(*appctypes.MustACIdentifier(key), val)
-		}
-	}
-
-	addLabel("layer", layerData.ID)
-	addLabel("version", dockerURL.Tag)
-	addLabel("os", layerData.OS)
-	addParentLabel("os", layerData.OS)
-	addLabel("arch", layerData.Architecture)
-	addParentLabel("arch", layerData.OS)
-
-	addAnno("authors", layerData.Author)
+	setAnnotation(&annotations, "authors", layerData.Author)
 	epoch := time.Unix(0, 0)
 	if !layerData.Created.Equal(epoch) {
-		addAnno("created", layerData.Created.Format(time.RFC3339))
+		setAnnotation(&annotations, "created", layerData.Created.Format(time.RFC3339))
 	}
-	addAnno("docker-comment", layerData.Comment)
-	addAnno(common.AppcDockerOriginalName, dockerURL.OriginalName)
-	addAnno(common.AppcDockerRegistryURL, dockerURL.IndexURL)
-	addAnno(common.AppcDockerRepository, dockerURL.ImageName)
-	addAnno(common.AppcDockerImageID, layerData.ID)
-	addAnno(common.AppcDockerParentImageID, layerData.Parent)
+	setAnnotation(&annotations, "docker-comment", layerData.Comment)
+	setAnnotation(&annotations, common.AppcDockerOriginalName, dockerURL.OriginalName)
+	setAnnotation(&annotations, common.AppcDockerRegistryURL, dockerURL.IndexURL)
+	setAnnotation(&annotations, common.AppcDockerRepository, dockerURL.ImageName)
+	setAnnotation(&annotations, common.AppcDockerImageID, layerData.ID)
+	setAnnotation(&annotations, common.AppcDockerParentImageID, layerData.Parent)
+	setAnnotation(&annotations, common.AppcDockerManifestHash, manhash)
 
 	if dockerConfig != nil {
 		exec := getExecCommand(dockerConfig.Entrypoint, dockerConfig.Cmd)
@@ -238,7 +262,9 @@ func GenerateManifest(layerData types.DockerImageData, dockerURL *common.ParsedD
 		var env appctypes.Environment
 		for _, v := range dockerConfig.Env {
 			parts := strings.SplitN(v, "=", 2)
-			env.Set(parts[0], parts[1])
+                        if len(parts) == 2 {
+				env.Set(parts[0], parts[1])
+                        }
 		}
 		app := &appctypes.App{
 			Exec:             exec,
@@ -265,10 +291,10 @@ func GenerateManifest(layerData types.DockerImageData, dockerURL *common.ParsedD
 			return nil, err
 		}
 		if len(ep) > 0 {
-			addAnno(common.AppcDockerEntrypoint, ep)
+			setAnnotation(&annotations, common.AppcDockerEntrypoint, ep)
 		}
 		if len(cmd) > 0 {
-			addAnno(common.AppcDockerCmd, cmd)
+			setAnnotation(&annotations, common.AppcDockerCmd, cmd)
 		}
 
 		genManifest.App = app
@@ -292,7 +318,7 @@ func GenerateManifest(layerData types.DockerImageData, dockerURL *common.ParsedD
 
 		genManifest.Dependencies = append(genManifest.Dependencies, appctypes.Dependency{ImageName: *parentImageName, Labels: plbl})
 
-		addAnno(common.AppcDockerTag, dockerURL.Tag)
+		setAnnotation(&annotations, common.AppcDockerTag, dockerURL.Tag)
 	}
 
 	genManifest.Labels, err = appctypes.LabelsFromMap(labels)
@@ -310,15 +336,15 @@ func GenerateEmptyManifest(name string) (*schema.ImageManifest, error) {
 		return nil, err
 	}
 
-	labels := appctypes.Labels{
-		appctypes.Label{
-			Name:  *appctypes.MustACIdentifier("arch"),
-			Value: runtime.GOARCH,
-		},
-		appctypes.Label{
-			Name:  *appctypes.MustACIdentifier("os"),
-			Value: runtime.GOOS,
-		},
+	labelsMap := make(map[appctypes.ACIdentifier]string)
+	err = setOSArch(labelsMap, runtime.GOOS, runtime.GOARCH)
+	if err != nil {
+		return nil, err
+	}
+
+	labels, err := appctypes.LabelsFromMap(labelsMap)
+	if err != nil {
+		return nil, err
 	}
 
 	return &schema.ImageManifest{
@@ -329,7 +355,18 @@ func GenerateEmptyManifest(name string) (*schema.ImageManifest, error) {
 	}, nil
 }
 
-func GenerateManifestV22(name string, dockerURL *common.ParsedDockerURL, config *typesV2.ImageConfig, imageDigest string, lowerLayers []*schema.ImageManifest, debug log.Logger) (*schema.ImageManifest, error) {
+// GenerateManifestV22, given a large set of information (documented a couple
+// lines down), will produce an image manifest compliant with the Dockver V2.2
+// image spec
+func GenerateManifestV22(
+	name string, // The name of this image
+	manhash string, // The hash of this image's manifest
+	imageDigest string, // The digest of the image
+	dockerURL *common.ParsedDockerURL, // The parsed docker URL
+	config *typesV2.ImageConfig, // The image config
+	lowerLayers []*schema.ImageManifest, // A list of manifests for the lower layers
+	debug log.Logger, // The debug logger, for logging debug information
+) (*schema.ImageManifest, error) {
 	manifest, err := GenerateEmptyManifest(name)
 	if err != nil {
 		return nil, err
@@ -338,30 +375,18 @@ func GenerateManifestV22(name string, dockerURL *common.ParsedDockerURL, config
 	labels := manifest.Labels.ToMap()
 	annotations := manifest.Annotations
 
-	addLabel := func(key, val string) {
-		if key != "" && val != "" {
-			labels[*appctypes.MustACIdentifier(key)] = val
-		}
-	}
+	setLabel(labels, "version", dockerURL.Tag)
+	setOSArch(labels, config.OS, config.Architecture)
 
-	addAnno := func(key, val string) {
-		if key != "" && val != "" {
-			annotations.Set(*appctypes.MustACIdentifier(key), val)
-		}
-	}
+	setAnnotation(&annotations, "author", config.Author)
+	setAnnotation(&annotations, "created", config.Created)
 
-	addLabel("version", dockerURL.Tag)
-	addLabel("os", config.OS)
-	addLabel("arch", config.Architecture)
-
-	addAnno("author", config.Author)
-	addAnno("created", config.Created)
-
-	addAnno(common.AppcDockerOriginalName, dockerURL.OriginalName)
-	addAnno(common.AppcDockerRegistryURL, dockerURL.IndexURL)
-	addAnno(common.AppcDockerRepository, dockerURL.ImageName)
-	addAnno(common.AppcDockerImageID, imageDigest)
-	addAnno("created", config.Created)
+	setAnnotation(&annotations, common.AppcDockerOriginalName, dockerURL.OriginalName)
+	setAnnotation(&annotations, common.AppcDockerRegistryURL, dockerURL.IndexURL)
+	setAnnotation(&annotations, common.AppcDockerRepository, dockerURL.ImageName)
+	setAnnotation(&annotations, common.AppcDockerImageID, imageDigest)
+	setAnnotation(&annotations, "created", config.Created)
+	setAnnotation(&annotations, common.AppcDockerManifestHash, manhash)
 
 	if config.Config != nil {
 		innerCfg := config.Config
@@ -370,7 +395,9 @@ func GenerateManifestV22(name string, dockerURL *common.ParsedDockerURL, config
 		var env appctypes.Environment
 		for _, v := range innerCfg.Env {
 			parts := strings.SplitN(v, "=", 2)
-			env.Set(parts[0], parts[1])
+                        if len(parts) == 2 {
+				env.Set(parts[0], parts[1])
+                        }
 		}
 		manifest.App = &appctypes.App{
 			Exec:             exec,
@@ -393,10 +420,10 @@ func GenerateManifestV22(name string, dockerURL *common.ParsedDockerURL, config
 			return nil, err
 		}
 		if len(ep) > 0 {
-			addAnno(common.AppcDockerEntrypoint, ep)
+			setAnnotation(&annotations, common.AppcDockerEntrypoint, ep)
 		}
 		if len(cmd) > 0 {
-			addAnno(common.AppcDockerCmd, cmd)
+			setAnnotation(&annotations, common.AppcDockerCmd, cmd)
 		}
 	}
 
@@ -586,23 +613,40 @@ func writeACI(layer io.ReadSeeker, manifest schema.ImageManifest, curPwl []strin
 		if name == "./" {
 			return nil
 		}
-		t.Header.Name = path.Join("rootfs", name)
-		absolutePath := strings.TrimPrefix(t.Header.Name, "rootfs")
+		newName := path.Join("rootfs", name)
+		absolutePath := strings.TrimPrefix(newName, "rootfs")
 
 		if filepath.Clean(absolutePath) == "/dev" && t.Header.Typeflag != tar.TypeDir {
 			return fmt.Errorf(`invalid layer: "/dev" is not a directory`)
 		}
 
 		fileMap[absolutePath] = struct{}{}
-		if strings.Contains(t.Header.Name, "/.wh.") {
+		if strings.Contains(newName, "/.wh.") {
 			whiteouts = append(whiteouts, strings.Replace(absolutePath, ".wh.", "", 1))
 			return nil
 		}
-		if t.Header.Typeflag == tar.TypeLink {
-			t.Header.Linkname = path.Join("rootfs", t.Linkname())
-		}
 
-		if err := trw.WriteHeader(t.Header); err != nil {
+		newHeader := &tar.Header{
+			Typeflag:   t.Header.Typeflag,
+			Name:       newName,
+			Linkname:   t.Header.Linkname,
+			Size:       t.Header.Size,
+			Mode:       t.Header.Mode,
+			Uid:        t.Header.Uid,
+			Gid:        t.Header.Gid,
+			Uname:      t.Header.Uname,
+			Gname:      t.Header.Gname,
+			ModTime:    t.Header.ModTime,
+			AccessTime: t.Header.AccessTime,
+			ChangeTime: t.Header.ChangeTime,
+			Devmajor:   t.Header.Devmajor,
+			Devminor:   t.Header.Devminor,
+			Xattrs:     t.Header.Xattrs,
+		}
+		if t.Header.Typeflag == tar.TypeLink {
+			newHeader.Linkname = path.Join("rootfs", t.Linkname())
+		}
+		if err := trw.WriteHeader(newHeader); err != nil {
 			return err
 		}
 		if _, err := io.Copy(trw, t.TarStream); err != nil {
